;;; ModelineCubes --- The modeline related cubes for FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;;; Code:
(require 'fpkg)
(require 'fg42/cube)
(require 'fg42/cubes/iconspack)

(autoload-cube 'fg42/statusbar-default
               "modeline/statusbar-default.el"
               "The default StatusBar modeline.")

(defface fg42/-disabled-modeline-active-border
    '((t
       :background "#bd93f9" :height 0.1 :box nil))
    "A new face for modeline in active state."
    :group 'fg42)

(defface fg42/-disabled-modeline-dective-border
    '((t
       :background "#44475a" :height 0.1 :box nil))
    "A new face for modeline in active state."
    :group 'fg42)


(defcube fg42/disable-modeline-cube
  "This cube disables the default modeline and replaces it with a thin border"
  :title "Disable Mode line"
  :flag disable-modeline
  :flag-default nil
  (setq-default mode-line-format "")
  (let ((face-remaps (default-value 'face-remapping-alist)))
     (setf (alist-get 'mode-line face-remaps)
           'fg42/-disabled-modeline-active-border
           (alist-get 'mode-line-inactive face-remaps)
           'fg42/-disabled-modeline-dective-border
           (default-value 'face-remapping-alist) face-remaps)))


(defcube fg42/smart-mode-line-cube
  "Smart mode line is a pretty simple yet fantastic alternative
to Emacs modeline."
  :title "Smart modeline cube"
  :flag smart-mode-line
  :flag-default nil

  (fpkg/use smart-mode-line
    :straight (smart-mode-line :source melpa)
    :defer nil
    :init
    (progn
      (require 'smart-mode-line)
      (setq sml/theme 'respectful)
      (setq sml/no-confirm-load-theme t)
      (sml/setup))))


;; TODO: Break this into two cubes
(defcube fg42/modeline-cube
  "Modeline cube"
  :title "Mini modeline"
  :flag mini-mode-line
  :group modeline
  :flag-default nil

  (defun fg42/mini-modeline-setter (def)
    (if (null def)
        (let ((def-list (funcall def)))
          (setq-default mini-modeline-l-format (plist-get def-list :long))
          (setq-default mini-modeline-r-format (plist-get def-list :brief)))

      (setq-default
       mini-modeline-r-format
       '("%e"
         mode-line-front-space
         mode-line-mule-info
         mode-line-client
         mode-line-modified
         mode-line-remote
         mode-line-frame-identification
         mode-line-buffer-identification
         mode-line-position
         evil-mode-line-tag
         ;; (:eval (string-trim (format-mode-line mode-line-modes)))
         mode-line-misc-info ))))


  (fpkg/use mini-modeline
    :straight (mini-modeline :repo "kiennq/emacs-mini-modeline"
                             :host github
                             :type git)
    :after smart-mode-line
    :init
    (progn
      (require 'smart-mode-line)
      (setq mini-modeline-enhance-visual nil)
      (setq fg42/modeline-setter #'fg42/mini-modeline-setter))
    :defer nil
    :config
    (add-hook 'fg42/after-init-hook #'mini-modeline-mode)
    (add-hook 'fg42/after-initializing-theme-hook
              (lambda ()
                (custom-set-faces
                 '(mini-modeline-mode-line
                   ((((background light))
                     :background "#aa0000" :height 0.1 :box nil)
                    (t
                     :background "#bd93f9" :height 0.1 :box nil))))))))


(defcube fg42/statusbar-cube
  "Status bar replaces the default mode line aned merges with
the mini buffer to provide a minimal interface for displaying
valuable information."
  :title "Status bar cube"
  :flag status-bar
  :group modeline
  :flag-default nil


  (fg42/all-the-icons-cube)

  (defun fg42/statusbar-setter (def)
    (unless (null def)
        (setq-default fg42/statusbar-components
                      (plist-get (funcall def) :brief))
      (setq-default fg42/statusbar-components
                    (plist-get (funcall #'fg42/statusbar-default) :brief))))

  (setq fg42/modeline-setter #'fg42/statusbar-setter)

  (add-hook 'fg42/after-init-hook
            (lambda ()
              (require 'fg42/statusbar)
              (fg42/statusbar-mode t))))


(provide 'fg42/cubes/modeline)
;;; modeline.el ends here
