;;; Themes --- Theme library of FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 4.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; Cubes are the building blocks of any `FG42' editor.  Each `cube' is a
;; unit which defines different abilities in a deterministic and idempotent
;; way.  Cubes are composable and a composition of cubes creates an editor.
;;
;;; Code:
(eval-when-compile
  (require 'fpkg))
(require 'fg42/utils)

(defvar fg42/ui-hook ()
  "A hook that cubes can use via :ui-hook property.
It executes way before the rest of the cubes.")

(defvar fg42/before-initializing-theme-hook ()
  "The hook to plug any configuration to before initialize event of themes.")

(defvar fg42/after-initializing-theme-hook ()
  "The hook to plug any configuration to after initialize event of themes.")


(defmacro fg42/setup-theme (&rest body)
  "Run the BODY inside the FG42 theme setup context."
  `(progn
     ;; TODO: This is a bad practice. Find a better solution
     (run-hooks 'fg42/before-initializing-theme-hook)
     ,@body))


(defmacro use-theme! (name &rest body)
  "Create a function to setup a them with the given NAME and BODY to customize it."
  (declare (indent defun))
  (let ((fn-name (intern (format "fg42/setup-%s" (symbol-name name)))))
    `(defun ,fn-name ()
       (use! ,name
         "Setting up the ,name package."
         :init
         (fg42/setup-theme
          (require ',name)
          ,@body)))))


;; (use-theme! dracula-theme
;;   :init
;;   (fg42/setup-theme
;;    (require 'dracula-theme)
;;    (load-theme 'dracula t)
;;    (custom-theme-set-faces
;;     'dracula
;;     '(match ((t (:background "#44475a"))))
;;     '(all-the-icons-lgreen ((t (:background "#bd93f9"))))
;;     '(all-the-icons-faicon ((t (:background "#bd93f9"))))
;;     '(font-lock-comment-face ((t (:foreground "#8B9298"))))
;;     ;; This fixes lsp-ui-sideline issue
;;     '(lsp-ui-sideline-current-symbol ((t (:line-width -1 :foreground "#bd93f9"))))
;;     '(font-lock-comment-delimiter-face ((t (:foreground "#5B6268")))))
;;    (enable-theme 'dracula)
;;    (set-face-attribute 'region nil :background "#888")))


(use-theme! badwolf-theme
  :init
  (fg42/setup-theme
   (load-theme 'badwolf t)
   (custom-theme-set-faces
    'badwolf)
   (enable-theme 'badwolf)))


(use! base16-theme
  "Load base16 based themes in FG42."
  :config
  (load-theme 'base16-eighties t))


(provide 'fg42/themes)
;;; themes.el ends here
